import matplotlib
# matplotlib.use('Agg')
import numpy as np
import pylab
import glob

__author__ = 'cliburn'

def plot_panel(ts, vals, xlabel, ylabel, degree):
    xs = []
    ys = []
    for t, v in zip(ts, vals):
        pylab.scatter([t]*len(v), v)
        xs.extend([t]*len(v))
        ys.extend(list(v))
    zs = np.polyfit(xs, ys, degree)
    p = np.poly1d(zs)
    x = np.linspace(np.min(ts), np.max(ts), 100)
    pylab.plot(x, p(x), '-')
    pylab.xlabel(xlabel, fontsize=14)
    pylab.ylabel(ylabel, fontsize=14)

if __name__ == '__main__':

    # original data sets used for timing are not included in the supp materials due to excessive size

    pylab.figure(figsize=(12,4))

    pylab.subplot(131)
    fs = sorted(glob.glob('../data/xss_n*'))
    ts = map(int, [f.split('_')[1][1:] for f in fs])
    ts = [t/1000 for t in ts]
    vals = [np.loadtxt(f) for f in fs]
    vals = [v/60.0 for v in vals]
    plot_panel(ts, vals, r'Number of events ($\times$ 1000)', 'Time (mins)', 1)
    pylab.xlim([0, 550])

    pylab.subplot(132)
    fs = sorted(glob.glob('../data/xss_s*'))
    ts = map(int, [f.split('_')[1][1:] for f in fs])
    vals = [np.loadtxt(f) for f in fs]
    vals = [v/60.0 for v in vals]
    plot_panel(ts, vals, 'Number of samples', '', 1)
    pylab.xlim([0, 33])

    pylab.subplot(133)
    fs = sorted(glob.glob('../data/xss_c*_ctime*'))
    ts = map(int, [f.split('_')[1][1:] for f in fs])
    vals = [np.loadtxt(f) for f in fs]
    vals = [v/60.0 for v in vals]
    plot_panel(ts, vals, 'Number of markers', '', 2)

    pylab.tight_layout()
    pylab.savefig('../figs/benchmarks.pdf')
