import matplotlib
# matplotlib.use('Agg')
import os, sys, glob
import numpy as np
import numpy.random as npr
import fcm
import fcm.statistics as stats
from fcm.graphics import bilinear_interpolate
import utils_cc as utils
import pylab
import copy

def dp_modal_cluster(xs, nclusts, niter, burnin,
                     tune_interval=100, verbose=True,
                     device=1, prior=None, e0=None, f0=None):

    m = stats.DPMixtureModel(nclusts, niter, burnin=burnin)
    m.device = device
    m.ident = True
    m.parallel = True
    if e0:
        m.e0 = e0
    if f0:
        m.f0 = f0
    if prior:
        m.prior_pi = prior.pis
        m.prior_mu = prior.mus
        m.prior_sigma = prior.sigmas
    rss = m.fit(xs, verbose=verbose)

    rs = [r.average() for r in rss]
    cs = [r.make_modal() for r in rs]

    return rss, rs, cs

if __name__ == '__main__':
    tune_interval = 100
    verbose = 10
    device = [int(_) for _ in sys.argv[1]]
    seed = int(sys.argv[2])
    nclusts = int(sys.argv[3])
    niter = int(sys.argv[4])
    burnin = int(sys.argv[5])

    base = 'dp'
    base_dir = '../data'

    identifier = '%s_k%d_i%d_b%d_s%d' % (base, nclusts, niter, burnin, seed)
    print identifier

    if os.path.exists('../results/%s_cs.pic.gz' % identifier):
        sys.exit(0)

    fs = sorted(glob.glob('%s/*xs.npy' % base_dir))
    xs = [np.load(f) for f in fs]

    npr.seed(seed)

    rss, rs, cs = dp_modal_cluster(xs, nclusts, niter, burnin,
                                   tune_interval, verbose, device)

    utils.pickle('../results/%s_rss.pic.gz' % identifier, rss)
    utils.pickle('../results/%s_rs.pic.gz' % identifier, rs)
    utils.pickle('../results/%s_cs.pic.gz' % identifier, cs)

