import matplotlib
matplotlib.use('Agg')
import numpy
import pylab
import fcm.statistics as stats
import utils_cc as utils
from matplotlib.colors import LinearSegmentedColormap
import sys

def contour(data, r, dim=[0,1], levels=20, nbins=100, log=True, fill=True):
    # minx, miny = numpy.min(data[:,dim], 0)
    # maxx, maxy = numpy.max(data[:,dim], 0)
    minx = miny = 0
    maxx = maxy = 14
    x = numpy.linspace(minx, maxx, nbins)
    y = numpy.linspace(miny, maxy, nbins)
    grid = numpy.array([(_x, _y) for _x in x for _y in y])
    X, Y = numpy.meshgrid(x, y)
    Z = stats.distributions.mixnormpdf(grid, r.pis, r.mus, r.sigmas)
    Z = Z.reshape((len(x), len(y)))
    if log:
        Z = numpy.log(Z)

    if numpy.max(Z) < 0.5:
        Z *= 100.0
    print numpy.min(Z), numpy.max(Z), numpy.mean(Z)
    if fill:
        pylab.contourf(Y, X, Z, levels)
    else:
        pylab.contour(Y, X, Z, levels)


def ind_modal_cluster(xs, nclusts, niter, burnin, ref=None, archive=None, verbose=True, background=0.005):
    m = stats.DPMixtureModel(nclusts, niter, burnin=burnin)
    m.ident = True
    m.device = 0
    rs = [m.fit(xs[i], verbose=verbose) for i in range(len(xs))]
    rs = [r.average() for r in rs]
    cs = [r.make_modal() for r in rs]
    # set reference cmap
    if ref is None:
        num_modes = []
        for i, _c in enumerate(cs):
            nm = 0
            zc = _c.classify(xs[i], usegpu=False)
            print background * len(zc)
            for j in numpy.unique(zc):
                if numpy.sum(zc==j) > (background * len(zc)):
                    nm += 1
            num_modes.append(nm)
            print i, nm
        ref = numpy.argmax(num_modes)

    if archive:
        # utils.pickle('%s_modal_xs.pic.gz' % archive, xs)
        utils.pickle('../results/%s_modal_rs.pic.gz' % archive, rs)
        utils.pickle('../results/%s_modal_cs.pic.gz' % archive, cs)
        utils.pickle('../results/%s_modal_ref.pic.gz' % archive, ref)

    return ref, rs, cs

def ref_modal_cluster(xs, nclusts, niter, burnin, ref=None, archive=None, verbose=True, background=0.005):
    m = stats.DPMixtureModel(nclusts, niter, burnin=burnin)
    m.ident = True
    m.device = 0
    rs = [m.fit(xs[i], verbose=verbose) for i in range(len(xs))]
    rs = [r.average() for r in rs]
    cs = [r.make_modal() for r in rs]
    # set reference cmap
    if ref is None:
        num_modes = []
        for i, _c in enumerate(cs):
            nm = 0
            zc = _c.classify(xs[i], usegpu=False)
            print background * len(zc)
            for j in numpy.unique(zc):
                if numpy.sum(zc==j) > (background * len(zc)):
                    nm += 1
            num_modes.append(nm)
            print i, nm
        ref = numpy.argmax(num_modes)

    print "ref"
    ref = 2

    ref_cmap = cs[ref].cmap
    for i in range(len(cs)):
        cs[i].cmap = ref_cmap

    if archive:
        # utils.pickle('%s_modal_xs.pic.gz' % archive, xs)
        utils.pickle('../results/%s_modal_rs.pic.gz' % archive, rs)
        utils.pickle('../results/%s_modal_cs.pic.gz' % archive, cs)
        utils.pickle('../results/%s_modal_ref.pic.gz' % archive, ref)

    return ref, rs, cs

def pool_modal_cluster(xs, nclusts, niter, burnin, ref=None, archive=None, verbose=True, background=0.005):
    m = stats.DPMixtureModel(nclusts, niter, burnin=burnin)
    m.ident = True
    m.device = 0
    rs = [m.fit(numpy.concatenate(xs), verbose=verbose)]
    rs = [r.average() for r in rs]
    cs = [r.make_modal() for r in rs]
    # set reference cmap
    if ref is None:
        num_modes = []
        for i, _c in enumerate(cs):
            nm = 0
            zc = _c.classify(xs[i], usegpu=False)
            print background * len(zc)
            for j in numpy.unique(zc):
                if numpy.sum(zc==j) > (background * len(zc)):
                    nm += 1
            num_modes.append(nm)
            print i, nm
        ref = numpy.argmax(num_modes)
    ref_cmap = cs[ref].cmap
    for i in range(len(cs)):
        cs[i].cmap = ref_cmap

    if archive:
        # utils.pickle('%s_modal_xs.pic.gz' % archive, xs)
        utils.pickle('../results/%s_modal_rs.pic.gz' % archive, rs)
        utils.pickle('../results/%s_modal_cs.pic.gz' % archive, cs)
        utils.pickle('../results/%s_modal_ref.pic.gz' % archive, ref)

    return ref, rs, cs

def hdp_modal_cluster(xs, nclusts, niter, burnin, ref=None, archive=None, verbose=True, background=0.005):
    """Performs the following sequence of events:
    1. Fits an HDP model to the list of dataset in xs
    2. Performs modal clustering on each dataset
    3. Finds the dataset with the greatest number of signfiicatn (>background frequency) modes as reference
    4. Sets reference cmap as common cmap for aligned modal clusters across data sets
    5. Returns the reference (ref), list of fitted comonnent restuls (rs) and list of fitted modal results (cs)
    """
    m = stats.HDPMixtureModel(nclusts, niter, burnin=burnin)
    m.ident = True
    m.device = 0
    rs = m.fit(xs, verbose=verbose)
    rs = [r.average() for r in rs]
    cs = [r.make_modal() for r in rs]

    # set reference cmap by averaging
    r_consensus = rs[0]
    pis = numpy.sum([r.pis for r in rs], 0)
    pis /= numpy.sum(pis)
    for i, cluster in enumerate(r_consensus.clusters):
        cluster.pi = pis[i]
    c_consensus = r_consensus.make_modal()
    ref_cmap = c_consensus.cmap

    for i in range(len(cs)):
        cs[i].cmap = ref_cmap

    if archive:
        # utils.pickle('%s_hdp_modal_xs.pic.gz' % archive, xs)
        utils.pickle('../results/%s_modal_rs.pic.gz' % archive, rs)
        utils.pickle('../results/%s_modal_cs.pic.gz' % archive, cs)
        utils.pickle('../results/%s_modal_ref.pic.gz' % archive, ref)

    ref = -1
    cs.append(c_consensus)
    return ref, rs, cs

if __name__ == '__main__':
    print "start"

    numpy.random.seed(23)
    nclust = 16
    burnin = int(sys.argv[1])
    niter = int(sys.argv[2])
    verbose = 10

    xs = utils.unpickle('../results/hdp_modal_xs_consensus.pic.gz')
    print "data uploaded"

    archive = 'sims_consensus_k%03d_n%06d_ind' % (nclust, niter)
    try:
        rs_ind = utils.unpickle('../results/%s_modal_rs.pic.gz' % archive)
        cs_ind = utils.unpickle('../results/%s_modal_cs.pic.gz' % archive)
        ref_ind = utils.unpickle('../results/%s_modal_ref.pic.gz' % archive)
    except Exception, e:
        ref_ind, rs_ind, cs_ind = ind_modal_cluster(xs, nclust, niter, burnin, archive=archive, verbose=verbose)

    print"%s done" % archive

    archive = 'sims_consensus_k%03d_n%06d_ref' % (nclust, niter)
    try:
        rs_ref = utils.unpickle('../results/%s_modal_rs.pic.gz' % archive)
        cs_ref = utils.unpickle('../results/%s_modal_cs.pic.gz' % archive)
        ref_ref = utils.unpickle('../results/%s_modal_ref.pic.gz' % archive)
    except Exception, e:
        ref_ref, rs_ref, cs_ref = ref_modal_cluster(xs, nclust, niter, burnin, archive=archive, verbose=verbose)
        # ref_ref = 3

    print"%s done" % archive

    archive = 'sims_consensus_k%03d_n%06d_pool' % (nclust, niter)
    try:
        rs_pool = utils.unpickle('../results/%s_modal_rs.pic.gz' % archive)
        cs_pool = utils.unpickle('../results/%s_modal_cs.pic.gz' % archive)
        ref_pool = utils.unpickle('../results/%s_modal_ref.pic.gz' % archive)
    except Exception, e:
        ref_pool, rs_pool, cs_pool = pool_modal_cluster(xs, nclust, niter, burnin, archive=archive, verbose=verbose)

    print"%s done" % archive

    archive = 'sims_consensus_k%03d_n%06d_hdp' % (nclust, niter)
    try:
        rs_hdp = utils.unpickle('../results/%s_modal_rs.pic.gz' % archive)
        cs_hdp = utils.unpickle('../results/%s_modal_cs.pic.gz' % archive)
        ref_hdp = utils.unpickle('../results/%s_modal_ref.pic.gz' % archive)
    except Exception, e:
        ref_hdp, rs_hdp, cs_hdp = hdp_modal_cluster(xs, nclust, niter, burnin, archive=archive, verbose=verbose)
        # ref_hdp = 3

    print"%s done" % archive
    methods = ['Individual', 'Reference', 'Pooled', 'Hierarchical']

    pylab.figure(figsize=(12,12))
    for k, (ref, rs, cs) in enumerate([[ref_ind, rs_ind, cs_ind],
                                       [ref_ref, rs_ref, cs_ref],
                                       [ref_pool, rs_pool, cs_pool],
                                       [ref_hdp, rs_hdp, cs_hdp]]):
        print "plotting row", k, ref

        colorsc = ['red', 'green', 'blue', 'orange', 'brown', 'grey', 'yellow', 'lightcoral', 'indigored', 'firebrick', 'salmon', 'coral', 'sieena', 'peru', 'moccasin', 'lime']

        for i, x in enumerate(xs):
            if k==0:
                cidx = midx = i
            elif k==1:
                cidx = 2
            elif k==3:
                cidx = i
                midx = -1
            else:
                cidx = midx = ref

            zc = cs[cidx].classify(x, usegpu=False)
            czc = [colorsc[_] for _ in zc]

            pylab.subplot(4,len(xs),k*len(xs)+i+1)
            contour(x, rs[cidx], dim=[0,1], levels=30, nbins=100, fill=True)

            pylab.axis([0,14,0,14])
            pylab.xticks([])
            pylab.yticks([])
            if k==3:
                pylab.xlabel('Data Set %d' % (i+1))
            if i==0:
                pylab.ylabel(methods[k])

    pylab.savefig('../figs/%s_hdp_modalcontour_log.png' % 'combined_consensus')
    pylab.clf()

    for k, (ref, rs, cs) in enumerate([[ref_ind, rs_ind, cs_ind],
                                       [ref_ref, rs_ref, cs_ref],
                                       [ref_pool, rs_pool, cs_pool],
                                       [ref_hdp, rs_hdp, cs_hdp]]):
        print "plotting row", k, ref

        colorsc = ['red', 'green', 'blue', 'orange', 'brown']
        for i, x in enumerate(xs):
            if k==0:
                cidx = midx = i
            elif k==1:
                cidx = 2
            elif k==3:
                cidx = i
                midx = -1
            else:
                cidx = midx = ref

            zc = cs[cidx].classify(x, usegpu=False)
            czc = [colorsc[_] for _ in zc]

            pylab.subplot(4,len(xs),k*len(xs)+i+1)
            pylab.scatter(x[:,0], x[:,1], c=czc, s=3, edgecolor='none')

            pylab.axis([0,14,0,14])
            pylab.xticks([])
            pylab.yticks([])
            if k==3:
                pylab.xlabel('Data Set %d' % (i+1))
            if i==0:
                pylab.ylabel(methods[k])

    pylab.savefig('../figs/%s_hdp_modalscatter.png' % 'combined_consensus')
    pylab.close()
