'''
Created on May 30, 2013

@author: Jacob Frelinger
'''
import matplotlib.pyplot as plt
import sys
import numpy as np
import glob
from matplotlib.nxutils import points_inside_poly
from fcm.graphics import bilinear_interpolate as blt

data_names = ['CD3CD45_0.000000_0_xs.npy', 'CD3CD45_0.015625_1_xs.npy',
              'CD3CD45_0.031250_2_xs.npy', 'CD3CD45_0.062500_3_xs.npy',
              'CD3CD45_0.125000_4_xs.npy', 'CD3CD45_0.250000_5_xs.npy']

nfiles = len(data_names)
nevents = 50000
cols = [0, 1, 2, 3, 6]
hs = ['FSC', 'SSC', 'CD45', 'CD3', 'Multimer']

dims = [(0, 1), (3, 2), (4, 2)]
percents = ['0.0%', '0.01562%', '0.03125%', '0.0625%', '0.125%', '0.25%']

n = len(dims)

base_dir = '../'
figs_dir = base_dir+'figs/'

gate1 = np.load('%sdata/gate_0_1.npy' % base_dir)
gate2 = np.load('%sdata/gate_3_2.npy' % base_dir)
gate3 = np.load('%sdata/gate_4_2.npy' % base_dir)
gates = [gate1, gate2, gate3]


def plot(xs, zs, tets, manual=False):
    fig = plt.figure(figsize=(3 * 2, 6 * 2))
    for idx, x in enumerate(xs):
        print idx, data_names[idx]
        for plot_idx, dim in enumerate(dims):
            subplot_idx = plot_idx + 1 + (idx * 3)
            ax = fig.add_subplot(6, 3, subplot_idx)
            ax.scatter(x[:, dim[0]], x[:, dim[1]], s=1, edgecolors='none', c=zs[idx])
            if np.any(tets[idx]):
                ax.scatter(x[tets[idx], dim[0]], x[tets[idx], dim[1]], s=10, c='red', edgecolor='black')
            ax.set_xlabel(hs[dim[0]], fontsize=14)
            ax.set_ylabel(hs[dim[1]], fontsize=14)
            ax.axis([0, 1024, 0, 1024])
            ax.set_xticks([])
            ax.set_yticks([])

            if plot_idx == 0:
                ax.text(100, 900, percents[idx], bbox=dict(facecolor='yellow', alpha=0.5))
            if plot_idx == 2:
                freq = float(np.sum(tets[idx])) / x.shape[0]
                ax.text(100, 900, str(freq), bbox=dict(facecolor='yellow', alpha=0.5))

    fig.tight_layout()
    return fig

def load_xs():
    xs = []
    fdir = '../data/'
    for i in range(6):
        xs.append(np.load(fdir + data_names[i]))
    return xs

def run_test():
    xs = load_xs()
    zs = np.random.randint(1, 6, size=nfiles * nevents).reshape(nfiles, nevents)
    tets = np.zeros((nfiles, nevents), dtype=np.bool)
    tets[:, -100:] = True
    fig = plot(xs, zs, tets)

    fig.savefig('test.png')

def run_flame48():
    flame_path = base_dir + 'data/flame48/'
    #load xs and zs
    xs = []
    zs = []
    flame_files = ['CD3CD45_0.000000_0_xs.mst.44.membership.txt',
                   'CD3CD45_0.015625_1_xs.mst.44.membership.txt',
                   'CD3CD45_0.031250_2_xs.mst.45.membership.txt',
                   'CD3CD45_0.062500_3_xs.mst.46.membership.txt',
                   'CD3CD45_0.125000_4_xs.mst.46.membership.txt',
                   'CD3CD45_0.250000_5_xs.mst.44.membership.txt']
    for i in flame_files:
        tmp = np.loadtxt(flame_path + i, skiprows=1)
        xs.append(tmp[:, [0, 1, 2, 3, 4]])
        zs.append(tmp[:, 5] - 1)
    #assign tetramer positives
    tets = []
    for j, i in enumerate(flame_files):
        i = i.replace('membership', 'locations')
        tmp = np.loadtxt(flame_path + i, skiprows=1)
        target_idx = (points_inside_poly(tmp[:, dims[0]], gates[0].T) &
                      points_inside_poly(tmp[:, dims[1]], gates[1].T) &
                      points_inside_poly(tmp[:, dims[2]], gates[2].T))
        values = np.arange(tmp.shape[0])[target_idx]
        tets.append(np.array([True if k in values else False for k in zs[j]]))

    #tets = np.zeros((nfiles,nevents), dtype=np.bool)
    fig = plot(xs, zs, tets)

    fig.savefig(figs_dir+'flame48.png')

def run_flame():
   flame_path = base_dir + 'data/flame/'
   #load xs and zs
   xs = []
   zs = []
   flame_files = ['matched.CD3CD45_0.000000_0_xs.mst.9.aligned.member.txt',
                  'matched.CD3CD45_0.015625_1_xs.mst.9.aligned.member.txt',
                  'matched.CD3CD45_0.031250_2_xs.mst.9.aligned.member.txt',
                  'matched.CD3CD45_0.062500_3_xs.mst.9.aligned.member.txt',
                  'matched.CD3CD45_0.125000_4_xs.mst.9.aligned.member.txt',
                  'matched.CD3CD45_0.250000_5_xs.mst.9.aligned.member.txt']
   for i in flame_files:
       tmp = np.loadtxt(flame_path + i, skiprows=1)
       xs.append(tmp[:, [0, 1, 2, 3, 4]])
       zs.append(tmp[:, 5] - 1)
   #assign tetramer positives
   tets = []
   for j, i in enumerate(flame_files):
       i = i.replace('member', 'locations')
       tmp = np.loadtxt(flame_path + i, skiprows=1)
       target_idx = (points_inside_poly(tmp[:, dims[0]], gates[0].T) &
                     points_inside_poly(tmp[:, dims[1]], gates[1].T) &
                     points_inside_poly(tmp[:, dims[2]], gates[2].T))
       values = np.arange(tmp.shape[0])[target_idx]
       tets.append(np.array([True if k in values else False for k in zs[j]]))

   #tets = np.zeros((nfiles,nevents), dtype=np.bool)
   fig = plot(xs, zs, tets)

   fig.savefig(figs_dir + 'flame.png')

def run_flowclust48():
    xs = load_xs()
    zs = []
    tets = []
    for j, i in enumerate(sorted(glob.glob(base_dir + 'data/flowclust48/*.labels'))):
        tmp = np.loadtxt(i, skiprows=1, usecols=[1], delimiter=',') - 1
        zs.append(tmp)
        i = i.replace('labels', 'mus')
        tmp = np.loadtxt(i, skiprows=1, usecols=[1, 2, 3, 4, 5], delimiter=',')
        target_idx = (points_inside_poly(tmp[:, dims[0]], gates[0].T) &
                      points_inside_poly(tmp[:, dims[1]], gates[1].T) &
                      points_inside_poly(tmp[:, dims[2]], gates[2].T))
        values = np.arange(tmp.shape[0])[target_idx]
        tets.append(np.array([True if k in values else False for k in zs[j]]))
    fig = plot(xs, zs, tets)

    fig.savefig(figs_dir + 'flowClust48.png')

def run_flowclust():
    xs = load_xs()
    zs = []
    tets = []
    for j, i in enumerate(sorted(glob.glob(base_dir + 'data/flowclust/*.labels'))):
        tmp = np.loadtxt(i, skiprows=1, usecols=[1], delimiter=',') - 1
        zs.append(tmp)
        i = i.replace('labels', 'mus')
        tmp = np.loadtxt(i, skiprows=1, usecols=[1, 2, 3, 4, 5], delimiter=',')
        target_idx = (points_inside_poly(tmp[:, dims[0]], gates[0].T) &
                      points_inside_poly(tmp[:, dims[1]], gates[1].T) &
                      points_inside_poly(tmp[:, dims[2]], gates[2].T))
        values = np.arange(tmp.shape[0])[target_idx]
        tets.append(np.array([True if k in values else False for k in zs[j]]))
    fig = plot(xs, zs, tets)

    fig.savefig(figs_dir + 'flowClust.png')

def run_flock():
    xs = load_xs()
    zs = []
    tets = []
    for j, i in enumerate(sorted(glob.glob(base_dir + 'data/flock/flock*zs.npy'))):
        zs.append(np.load(i) - 1)
        i = i.replace('zs', 'cx')
        tmp = np.load(i)[:, 1:]
        target_idx = (points_inside_poly(tmp[:, dims[0]], gates[0].T) &
                      points_inside_poly(tmp[:, dims[1]], gates[1].T) &
                      points_inside_poly(tmp[:, dims[2]], gates[2].T))
        values = np.arange(tmp.shape[0])[target_idx]
        tets.append(np.array([True if k in values else False for k in zs[j]]))
    fig = plot(xs, zs, tets)

    fig.savefig(figs_dir+'flock.png')



load_dict = {}
#load_dict['manual'] = run_manual
#load_dict['dp'] = run_dp
#load_dict['hdp'] = run_hdp
load_dict['flame'] = run_flame
load_dict['flame48'] = run_flame48
load_dict['flock'] = run_flock
load_dict['flowclust'] = run_flowclust
load_dict['flowclust48'] = run_flowclust48
load_dict['test'] = run_test

if __name__ == '__main__':
    plot_type = sys.argv[1]
    if plot_type not in load_dict:
        print "Plot type not understood - %s" % plot_type
        print 'Please chose one of:'
        for i in load_dict.keys():
            print '  ', i
    else:
        load_dict[plot_type]()
