import matplotlib
matplotlib.use('Agg')
import os, sys, glob
import numpy as np
import numpy.random as npr
import fcm
import fcm.statistics as stats
from fcm.graphics import bilinear_interpolate
import utils_cc as utils
import pylab
from matplotlib.nxutils import points_inside_poly

if __name__ == '__main__':
    tune_interval = 100
    verbose = 10
    device = [int(_) for _ in sys.argv[1]]
    seed = int(sys.argv[2])
    nclusts = int(sys.argv[3])
    niter = int(sys.argv[4])
    burnin = int(sys.argv[5])
    try:
        test_cluster = int(sys.argv[6])
    except:
        test_cluster = None

    base = 'hdp'
    base_dir = '../data'

    identifier = '%s_k%d_i%d_b%d_s%d' % (base, nclusts, niter, burnin, seed)
    print identifier

    plot_type = 'mode'
    summary = 'average'

    if os.path.exists('../figs/%s_%s_%s_target_nogate.png' %
                      (identifier, summary, plot_type)):
        sys.exit(0)

    rs = utils.unpickle('../results/%s_rs.pic.gz' % identifier)
    cs = utils.unpickle('../results/%s_cs.pic.gz' % identifier)

    fs = sorted(glob.glob('%s/*xs.npy' % base_dir))
    xs = [np.load(f) for f in fs]
    markers = ['FSC', 'SSC', 'CD45', 'CD3', 'Multimer']

    gate1 = np.load('%s/gate_0_1.npy' % base_dir)
    gate2 = np.load('%s/gate_3_2.npy' % base_dir)
    gate3 = np.load('%s/gate_4_2.npy' % base_dir)
    gates = [gate1, gate2, gate3]

    dims = [(0,1), (3,2), (4,2)]
    nrows = len(xs)
    ncols = len(dims)
    pylab.figure(figsize=(2*ncols, 2*nrows))

    if summary == 'average':
        rx = rs
        cx = cs

    modes = cx[-1].modes()
    target_idx = (points_inside_poly(modes[:,dims[0]], gates[0].T) &
                  points_inside_poly(modes[:,dims[1]], gates[1].T) &
                  points_inside_poly(modes[:,dims[2]], gates[2].T))
    target = np.arange(len(modes))[target_idx]
    print target

    ps = []
    for j, x in enumerate(xs):
        pdf = np.sum(rx[j].prob(x), 1)
        ipdf = np.argsort(pdf)

        z = cx[j].classify(x, use_gpu=False)
        iz = np.argsort(z)
        centroids = cx[j].modes()

        p = 0.0
        if target is not None:
            for t in target:
                p += np.sum(z==t)
        p = 100.0*p/len(z)
        ps.append(p)

        for k, dim in enumerate(dims):
            pylab.subplot(nrows, ncols, j*ncols+k+1)

            pylab.scatter(x[iz, dim[0]], x[iz, dim[1]],
                          s=1, c=z[iz], edgecolors='none')
            if target is not None:
                for t in target:
                    pylab.scatter(x[z==t, dim[0]], x[z==t, dim[1]],
                                  s=10, c='red', edgecolors='black')

            if (target is not None) and k==2:
                pylab.text(50, 900, '%.4f%%' % p,
                           bbox=dict(facecolor='yellow', alpha=0.5))

            ## Uncomment to show gates
            # pylab.fill(gates[k][0], gates[k][1], facecolor='none',
            #            edgecolor='red')

            pylab.axis([0,1024,0,1024])
            pylab.xticks([])
            pylab.yticks([])
            pylab.xlabel(markers[dim[0]])
            pylab.ylabel(markers[dim[1]])
    pylab.tight_layout()

    pylab.savefig('../figs/%s_%s_%s_target_nogate.png' %
                  (identifier, summary, plot_type))
    np.save('../results/%s_%s_%s_target_.npy' % (identifier, summary, plot_type), ps)
