import matplotlib
# matplotlib.use('Agg')
import os, sys, glob
import numpy as np
import numpy.random as npr
import fcm
import fcm.statistics as stats
from fcm.graphics import bilinear_interpolate
import utils_cc as utils
import pylab
from scipy import linspace, polyval, polyfit, sqrt, stats, randn

# these parameters must match those in the generate_figures.sh script
nclusts = int(sys.argv[1])
niter = int(sys.argv[2])
burnin = int(sys.argv[3])
ident = 'k%d_i%d_b%d' % (nclusts, niter, burnin)

x_dp = [np.load(f) for f in
         sorted(glob.glob('../results/dp_%s_s*_average_mode_target_.npy' % ident))]
x_hdp = [np.load(f) for f in
         sorted(glob.glob('../results/hdp_%s_s*_average_mode_target_.npy' % ident))]
x_man = np.loadtxt('../data/manual_counts.txt')/500.0

c = np.array([0, 0.013125, 0.02625, 0.0525, 0.105, 0.21])
background = x_man.mean(0)[0]

x0 = np.array(x_man)
man_acc = np.abs(np.mean(x0, 0) - (c + background))
man_cv = x0.std(0)/x0.mean(0)

x1 = np.array(x_dp)
dp_acc = np.abs(np.mean(x1, 0) - (c + background))
dp_cv = x1.std(0)/x0.mean(0)

x2 = np.array(x_hdp)
hdp_acc = np.abs(np.mean(x2, 0) - (c + background))
hdp_cv = x2.std(0)/x0.mean(0)

min_acc = np.min(np.vstack([man_acc, dp_acc, hdp_acc]), 0)

if __name__ == '__main__':

    labels = ['Manual', 'DPGMM', 'HDPGMM']
    pylab.figure(figsize=(15,6))
    for k, xs in enumerate([x_man, x_dp, x_hdp]):

        slope, intercept, r_value, p_value, std_err = stats.linregress(np.tile(c, (len(xs))),np.hstack(xs))

        pylab.subplot(1,3,k+1)
        mus = np.mean(xs, 0)
        for _c, _m in zip(c, mus):
            pylab.plot([_c-0.005, _c+0.005], [_m, _m], linewidth=2, color='blue')

        for x in xs:
            cr = c + npr.uniform(-0.002, 0.002, len(c))
            pylab.scatter(cr, x, marker='o', facecolor='none',
                          edgecolor='blue', s=50)


        line = slope*c+intercept
        pylab.plot(c,line,'r-')

        pylab.scatter(c, c + background, color='red', marker='x',
                      edgecolor='red', s=75, linewidth=2)

        pylab.xlim([-0.02, 0.23])
        pylab.ylim([-0.04, 0.31])
        pylab.xticks(c, c, rotation=90)
        if k==0:
            pylab.ylabel('Estimated frequency (%)', fontsize=18)
        if k==1:
            pylab.xlabel('Spiked frequency (%)', fontsize=18)

        if k==0:
            acc = man_acc
            cv = man_cv
        elif k==1:
            acc = dp_acc
            cv = dp_cv
        else:
            acc = hdp_acc
            cv = hdp_cv

        for i, _c in enumerate(c[1:]):
            pylab.text(_c, _c+background+0.03, '%.4f' % acc[i+1], ha='center', color='red')

        for i, _c in enumerate(c[:]):
            pylab.text(_c, line[i]-0.05, '%.4f' % cv[i], ha='center', color='blue')

        pylab.text(0.0, 0.29, labels[k],
                   va='center', ha='left', fontsize=14)
        pylab.text(0.0, 0.27, 'Correlation coefficient = %.3f' % r_value,
                   va='center', ha='left', fontsize=14)
        pylab.text(0.0, 0.25, 'Standard error = %.3f' % std_err,
                   va='center', ha='left', fontsize=14)

    pylab.tight_layout()
    pylab.savefig('../figs/regression.pdf')
